@@ -1,4 +1,9 @@
-import os
+# -*- coding: utf-8 -*-
+"""Plotting utilities.
+"""
+
+__author__ = ["patrickzib"]
+
 import time
 
 import matplotlib
@@ -8,7 +13,6 @@ import seaborn as sns
 from matplotlib import pyplot as plt
 from matplotlib.patches import Rectangle
 from matplotlib.ticker import MaxNLocator
-from numba import njit
 from scipy.stats import zscore
 
 import motiflets.motiflets as ml
@@ -16,14 +20,44 @@ import motiflets.motiflets as ml
 matplotlib.rcParams['pdf.fonttype'] = 42
 matplotlib.rcParams['ps.fonttype'] = 42
 
-# Store figures??
-save_fig = True
 
-def plot_dataset(name, data, ds_name, ground_truth=None):
-    plot_motifset(name, data, ds_name=ds_name, ground_truth=ground_truth)
+def plot_dataset(ds_name, data, ground_truth=None):
+    """Plots a time series.
+
+    Parameters
+    ----------
+    ds_name: String
+        The name of the time series
+    data: array-like
+        The time series
+    ground_truth: pd.Series
+        Ground-truth information as pd.Series.
+
+    """
+    plot_motifset(ds_name, data, ground_truth=ground_truth)
 
 
 def append_all_motif_sets(df, motif_sets, method_name, D_full):
+    """Utility function.
+
+    Parameters
+    ----------
+    df: pd.DataFrame
+        a dataframe to append to
+    motif_sets: 2d-array-like
+        The motif-sets to append under row `method_name`
+    method_name: String
+        The column to append as
+    D_full:
+        The distance matrix
+
+    Returns
+    -------
+    df: pd.DataFrame
+        the dataframe with appended data
+
+    """
+
     filtered_motif_sets = [m for m in motif_sets if m is not None]
     extent = [ml.get_pairwise_extent(D_full, motiflet) for motiflet in
               filtered_motif_sets]
@@ -36,12 +70,31 @@ def append_all_motif_sets(df, motif_sets, method_name, D_full):
 
 
 def plot_motifset(
-        name, data,
-        motifset=None, dist=None,
+        ds_name,
+        data,
+        motifset=None,
+        dist=None,
         motif_length=None,
-        idx=None, ds_name=None,
-        prefix="",
         ground_truth=None):
+    """Plots the data and the found motif sets.
+
+    Parameters
+    ----------
+    ds_name: String,
+        The name of the time series
+    data: array-like
+        The time series data
+    motifset: 2d-array like
+        The found motif set
+    dist: array like
+        The distances (extents) for each motif set
+    motif_length: int
+        The length of the motif
+    ground_truth: pd.Series
+        Ground-truth information as pd.Series.
+
+    """
+
     if motifset is not None:
         fig, axes = plt.subplots(1, 2, sharey=False,
                                  sharex=False, figsize=(20, 3),
@@ -53,14 +106,9 @@ def plot_motifset(
     if ground_truth is None:
         ground_truth = []
 
-    if isinstance(data, pd.Series):
-        data_raw = data.values
-        data_index = data.index
-    else:
-        data_raw = data
-        data_index = np.arange(len(data))
+    data_index, data_raw = ml._pd_series_to_numpy(data)
 
-    axes[0].set_title(ds_name if ds_name is not None else name, fontsize=20)
+    axes[0].set_title(ds_name, fontsize=20)
     _ = sns.lineplot(x=data_index, y=data_raw, ax=axes[0], linewidth=1)
     sns.despine()
 
@@ -71,11 +119,6 @@ def plot_motifset(
                              y=data_raw[pos:pos + motif_length], linewidth=5,
                              color=sns.color_palette()[len(ground_truth) + 2])
 
-    if isinstance(data, pd.Series):
-        data_raw = data.to_numpy()
-    else:
-        data_raw = data
-
     for aaa, column in enumerate(ground_truth):
         for offsets in ground_truth[column]:
             for pos, offset in enumerate(offsets):
@@ -112,33 +155,35 @@ def plot_motifset(
         motifset = []
 
     fig.tight_layout()
-
-    if save_fig:
-        dir_name = "../images/"
-        if idx is None:
-            plt.savefig(
-                dir_name + "/" + name + "_" + str(prefix) +
-                "_" + str(len(motifset)) + ".pdf", bbox_inches='tight')
-        else:
-            plt.savefig(
-                dir_name + "/" + name + "_" + str(prefix) +
-                "_" + str(len(motifset)) + "_" + str(idx) + ".pdf", bbox_inches='tight')
-
     plt.show()
 
 
-def plot_elbow_points(
-        name, data, motif_length, elbow_points, candidates, dists):
-    
-    if isinstance(data, pd.Series):
-        data_raw = data.values
-        data_index = data.index
-    else:
-        data_raw = data
-        data_index = np.arange(len(data))
-    
+def _plot_elbow_points(
+        ds_name, data, motif_length, elbow_points,
+        motifset_candidates, dists):
+    """Plots the elbow points found.
+
+    Parameters
+    ----------
+    ds_name: String
+        The name of the time series.
+    data: array-like
+        The time series data.
+    motif_length: int
+        The length of the motif.
+    elbow_points: array-like
+        The elbow points to plot.
+    motifset_candidates: 2d array-like
+        The motifset candidates. Will only extract those motif sets
+        within elbow_points.
+    dists: array-like
+        The distances (extents) for each motif set
+    """
+
+    data_index, data_raw = ml._pd_series_to_numpy(data)
+
     fig, ax = plt.subplots(figsize=(8, 4), constrained_layout=True)
-    ax.set_title(name + "\nElbow Points")
+    ax.set_title(ds_name + "\nElbow Points")
     ax.plot(range(2, len(np.sqrt(dists))), dists[2:], "b", label="Extent")
 
     lim1 = plt.ylim()[0]
@@ -151,11 +196,11 @@ def plot_elbow_points(
     ax.set(xlabel='Size (k)', ylabel='Extent')
     ax.xaxis.set_major_locator(MaxNLocator(integer=True))
 
-    motiflets = candidates[elbow_points]
+    motiflets = motifset_candidates[elbow_points]
     for i, motiflet in enumerate(motiflets):
         if motiflet is not None:
             axins = ax.inset_axes(
-                 [(elbow_points[i]-3) / (len(candidates)-2), 0.7, 0.3, 0.3])
+                [(elbow_points[i] - 3) / (len(motifset_candidates) - 2), 0.7, 0.3, 0.3])
 
             df = pd.DataFrame()
             df["time"] = data_index[range(0, motif_length)]
@@ -171,75 +216,127 @@ def plot_elbow_points(
             axins.xaxis.set_major_formatter(plt.NullFormatter())
             axins.yaxis.set_major_formatter(plt.NullFormatter())
 
-    if save_fig:
-        plt.savefig("../images/" + name.replace(" ", "-") + "_elbow.pdf", bbox_inches='tight')
     plt.show()
 
 
-def plot_elbow(ks,
+def plot_elbow(k_max,
                data,
-               dataset,
+               ds_name,
                motif_length,
-               ds_name=None,
                exclusion=None,
-               idx=None,
                plot_elbows=False,
                ground_truth=None,
                filter=True,
-               method_name=None,
-               data_array=None):
-    raw_data = data.to_numpy() if isinstance(data, pd.Series) else data
+               method_name=None):
+    """Plots the elbow-plot for k-Motiflets.
+
+    This is the method to find and plot the characteristic k-Motiflets within range
+    [2...k_max] for given a `motif_length` using elbow-plots.
+
+    Details are given within the paper Section 5.1 Learning meaningful k.
+
+    Parameters
+    ----------
+    k_max: int
+        use [2...k_max] to compute the elbow plot (user parameter).
+    data: array-like
+        the TS
+    ds_name: String
+        the name of the dataset
+    motif_length: int
+        the length of the motif (user parameter)
+    exclusion: 2d-array
+        exclusion zone - use when searching for the TOP-2 motiflets
+    plot_elbows: bool, default=False
+        plots the elbow ploints into the plot
+    ground_truth: pd.Series
+        Ground-truth information as pd.Series.
+    filter: bool, default=True
+        filters overlapping motiflets from the result,
+    method_name:  String
+        used for display only.
+
+    Returns
+    -------
+    Tuple
+        dists:          distances for each k in [2...k_max]
+        candidates:     motifset-candidates for each k
+        elbow_points:   elbow-points
+
+    """
+    _, raw_data = ml._pd_series_to_numpy(data)
     print("Data", len(raw_data))
 
     startTime = time.perf_counter()
     dists, candidates, elbow_points, m = ml.search_k_motiflets_elbow(
-        ks,
+        k_max,
         raw_data,
-        dataset,
         motif_length,
         exclusion=exclusion)
     endTime = (time.perf_counter() - startTime)
 
     print("Chosen window-size:", m, "in", np.round(endTime, 1), "s")
 
-    if exclusion is not None and idx is None:
-        idx = "top-2"
-
     if filter:
-        elbow_points = ml.filter_unqiue(elbow_points, candidates, motif_length)
-    
+        elbow_points = ml._filter_unique(elbow_points, candidates, motif_length)
+
     print("Elbow Points", elbow_points)
 
     if plot_elbows:
-        plot_elbow_points(ds_name, data, motif_length, elbow_points, candidates, dists)
+        _plot_elbow_points(ds_name, data, motif_length, elbow_points, candidates, dists)
 
     print("Data", len(data))
 
     plot_grid_motiflets(
-        dataset, data, candidates, elbow_points,
-        dists, motif_length, idx=idx, ds_name=ds_name, show_elbows=plot_elbows, font_size=24,
-        ground_truth=ground_truth, method_name=method_name, data_array=data_array)
+        ds_name, data, candidates, elbow_points,
+        dists, motif_length, show_elbows=plot_elbows,
+        font_size=24,
+        ground_truth=ground_truth, method_name=method_name)
 
     return dists, candidates, elbow_points
 
 
-def plot_motif_length_selection(ks, data, dataset, motif_length_range, ds_name):
-    # raw_data = data.values if isinstance(data, pd.Series) else data
-    index = data.index if isinstance(data, pd.Series) else np.arange(len(data))
-    header = " in " + data.index.name if isinstance(data, pd.Series) and data.index.name != None else ""
+def plot_motif_length_selection(k_max, data, motif_length_range, ds_name):
+    """Computes the AU_EF plot to extract the best motif lengths
+
+    This is the method to find and plot the characteristic motif-lengths, for k in
+    [2...k_max], using the area AU-EF plot.
+
+    Details are given within the paper 5.2 Learning Motif Length l.
+
+    Parameters
+    ----------
+    k_max: int
+        use [2...k_max] to compute the elbow plot.
+    data: array-like
+        the TS
+    motif_length_range: array-like
+        the interval of lengths
+    ds_name: String
+        Name of the time series for displaying
+
+    Returns
+    -------
+    best_motif_length: int
+        The motif length that maximizes the AU-PDF.
+
+    """
+    index, _ = ml._pd_series_to_numpy(data)
+    header = " in " + data.index.name if isinstance(data,
+                                                    pd.Series) and data.index.name != None else ""
 
     startTime = time.perf_counter()
-    best_motif_length, au_pdfs, elbow, top_motiflets = \
-        ml.find_au_pef_motif_length(
-            data, dataset, ks, motif_length_range=motif_length_range)
+    best_motif_length, au_ef, elbow, top_motiflets = \
+        ml.find_au_ef_motif_length(
+            data, k_max, motif_length_range=motif_length_range)
     endTime = (time.perf_counter() - startTime)
-    print ("\tTime", np.round(endTime, 1), "s")
-    
-    indices = ~np.isinf(au_pdfs)
+    print("\tTime", np.round(endTime, 1), "s")
+
+    indices = ~np.isinf(au_ef)
     fig, ax = plt.subplots(figsize=(4, 4))
     ax = sns.lineplot(
         x=index[motif_length_range[indices]],
-        y=au_pdfs[indices],
+        y=au_ef[indices],
         label="AU_EF")
     sns.despine()
     plt.tight_layout()
@@ -247,41 +344,70 @@ def plot_motif_length_selection(ks, data, dataset, motif_length_range, ds_name):
     ax.set(xlabel='Motif Length' + header, ylabel='Area under EF\n(lower is better)')
 
     for item in ([ax.xaxis.label, ax.yaxis.label] +
-             ax.get_xticklabels() + ax.get_yticklabels()):
+                 ax.get_xticklabels() + ax.get_yticklabels()):
         item.set_fontsize(16)
 
     # plt.legend(loc="best")      
     fig.set_figheight(5)
     fig.set_figwidth(5)
-    if save_fig:
-        plt.savefig(
-            "../images/" + ds_name.replace(" ", "-") + "_motif_length_selection.pdf",
-            bbox_inches='tight')
     plt.show()
 
     return best_motif_length
 
 
 def plot_grid_motiflets(
-        name, data, candidates, elbow_points, dist,
+        ds_name, data, candidates, elbow_points, dist,
         motif_length, font_size=20,
-        idx=None, ds_name=None,
         ground_truth=None,
         method_name=None,
         method_names=None,
         show_elbows=False,
         color_palette=sns.color_palette(),
         grid_dim=None,
-        plot_index=None,
-        data_array=None):
-    sns.set(font_scale = 2)
+        plot_index=None):
+    """Plots the characteristic motifs for each method along the time series.
+
+    Parameters
+    ----------
+    ds_name: String
+        The name of the time series
+    data: array-like
+        The time series data
+    candidates: 2d array-like
+        The motifset candidates
+    elbow_points: array-like
+        The elbow points found. Only motif sets from the elbow points will be plotted.
+    dist: array-like
+        The distances (extents) of the motif set candidates
+    motif_length: int
+        The motif length found.
+    font_size: int
+        Font-size to use for plotting.
+    ground_truth: pd.Series
+        Ground-truth information as pd.Series.
+    method_name: String
+        Name of a single method
+    method_names: array-like
+        Name of all methods
+    show_elbows: bool
+        Show an elbow plot
+    color_palette:
+        Color-palette to use
+    grid_dim: int
+        The dimensionality of the grid (number of columns)
+    plot_index: int
+        Plots only the passed methods in the given order
+
+    """
+
+    sns.set(font_scale=2)
     sns.set_style("white")
     sns.set_context("paper",
-                    rc={"font.size": font_size, 
+                    rc={"font.size": font_size,
                         "axes.titlesize": font_size - 8,
                         "axes.labelsize": font_size - 8,
                         "xtick.labelsize": font_size - 10,
-                        "ytick.labelsize": font_size - 10,})
+                        "ytick.labelsize": font_size - 10, })
 
     label_cols = 2
 
@@ -298,38 +424,20 @@ def plot_grid_motiflets(
         else:
             ll = len(elbow_points)
         grid_dim = int(max(2, np.ceil(ll / 2)))
-        
+
     dims = int(np.ceil(len(elbow_points) / grid_dim)) + count_plots
 
     fig = plt.figure(constrained_layout=True, figsize=(10, dims * 2))
     gs = fig.add_gridspec(dims, grid_dim, hspace=0.8, wspace=0.4)
 
     ax_ts = fig.add_subplot(gs[0, :])
-    ax_ts.set_title("(a) Dataset: " + (ds_name if ds_name is not None else name) + "")
+    ax_ts.set_title("(a) Dataset: " + ds_name + "")
 
-    if isinstance(data, pd.Series):
-        data_raw = data.values
-        data_index = data.index
-    else:
-        data_raw = data
-        data_index = np.arange(len(data))
-
-    if data_array is None:
-        _ = sns.lineplot(x=data_index, y=data_raw, ax=ax_ts, linewidth=1)
-
-    if data_array is not None:
-        last_offset = 0
-        for d in data_array:
-            d_data = d.values
-            d_index = np.arange(last_offset, last_offset+len(d_data))
-            _ = sns.lineplot(x=d_index, y=d_data, ax=ax_ts, label=d.name, linewidth=1)
-            last_offset = last_offset + len(d_data)
-
-        ax_ts.legend()
+    data_index, data_raw = ml._pd_series_to_numpy(data)
 
+    _ = sns.lineplot(x=data_index, y=data_raw, ax=ax_ts, linewidth=1)
     sns.despine()
 
-
     for aaa, column in enumerate(ground_truth):
         for offsets in ground_truth[column]:
             for pos, offset in enumerate(offsets):
@@ -348,16 +456,15 @@ def plot_grid_motiflets(
     if len(candidates[elbow_points]) > 6:
         ax_bars = fig.add_subplot(gs[1:3, :], sharex=ax_ts)
         next_id = 3
-    else: 
-        ax_bars = fig.add_subplot(gs[1, :], sharex=ax_ts)        
+    else:
+        ax_bars = fig.add_subplot(gs[1, :], sharex=ax_ts)
         next_id = 2
 
     ax_bars.set_title("(b) Position of Top Motif Sets")
 
     if show_elbows:
         ax_elbow = fig.add_subplot(gs[next_id, :])
-        ax_elbow.set_title("(c) Significant Elbow Points on " + (
-            ds_name if ds_name is not None else name))
+        ax_elbow.set_title("(c) Significant Elbow Points on " + ds_name)
         ax_elbow.plot(range(len(np.sqrt(dist))), dist, "b", label="Extent")
         lim1 = plt.ylim()[0]
         lim2 = plt.ylim()[1]
@@ -396,19 +503,19 @@ def plot_grid_motiflets(
         if motiflet is not None:
 
             plot_minature = (plot_index == None) or (i in plot_index)
-            if plot_minature:                
-                ii = ii+1
+            if plot_minature:
+                ii = ii + 1
                 off = int(ii / grid_dim)
-                ax_motiflet = fig.add_subplot(gs[count_plots + off, ii % grid_dim])                
-            
+                ax_motiflet = fig.add_subplot(gs[count_plots + off, ii % grid_dim])
+
             df = pd.DataFrame()
             df["time"] = data_index[range(0, motif_length)]
 
             for aa, pos in enumerate(motiflet):
                 df[str(aa)] = zscore(data_raw[pos:pos + motif_length])
-                ratio = 0.8 
+                ratio = 0.8
                 rect = Rectangle(
-                    (data_index[pos], -i),  
+                    (data_index[pos], -i),
                     data_index[pos + motif_length - 1] - data_index[pos],
                     ratio,
                     facecolor=color_palette[
@@ -437,9 +544,11 @@ def plot_grid_motiflets(
 
             if plot_minature:
                 df_melt = pd.melt(df, id_vars="time")
-                _ = sns.lineplot(ax=ax_motiflet, data=df_melt, x="time", y="value", ci=99,
+                _ = sns.lineplot(ax=ax_motiflet, data=df_melt, x="time", y="value",
+                                 ci=99,
                                  color=color_palette[
-                                     (len(ground_truth) + ii % grid_dim) % len(color_palette)],
+                                     (len(ground_truth) + ii % grid_dim) % len(
+                                         color_palette)],
                                  label=label + "k=" + str(len(motiflet)) + ",d=" + dists
                                  )
                 ax_motiflet.set_ylabel("")
@@ -483,9 +592,9 @@ def plot_grid_motiflets(
             if plot_minature:
                 ax_motiflet.set_yticks([])
 
-    ax_bars.set_yticks(-np.arange(len(y_labels)) + 0.5,)
+    ax_bars.set_yticks(-np.arange(len(y_labels)) + 0.5, )
     ax_bars.set_yticklabels(y_labels, fontsize=12)
-    ax_bars.set_ylim([-len(motiflets)+1, 1])
+    ax_bars.set_ylim([-len(motiflets) + 1, 1])
     # ax_bars.legend(loc="best")
 
     if (ground_truth is not None and len(ground_truth) > 0):
@@ -493,38 +602,42 @@ def plot_grid_motiflets(
 
     plt.tight_layout()
     gs.tight_layout(fig)
-
-    if method_names is not None and save_fig:
-        if idx is None:
-            plt.savefig("../images/" + name.replace(" ", "-") + "_grid.pdf",
-                        bbox_inches='tight')
-        else:
-            plt.savefig(
-                "../images/" + name.replace(" ", "-") + "_grid_" + str(idx) + ".pdf",
-                bbox_inches='tight')
-
-    if save_fig:
-        plt.savefig("../images/" + name.replace(" ", "-") + "_elbows.pdf",
-                    bbox_inches='tight')
-
     plt.show()
 
 
 def plot_all_competitors(
         data,
-        name,
+        ds_name,
         motifsets,
         motif_length,
         method_names=None,
-        target_cardinalities=None,
         ground_truth=None,
         plot_index=None,
         color_palette=sns.color_palette()):
-    # convert to numpy array
-    data_raw = data
-    if isinstance(data, pd.Series):
-        data_raw = data.to_numpy()
+    """Plots the found motif sets of multiple competitor methods
 
+    Parameters
+    ----------
+    ds_name: String
+        The name of the time series
+    data: array-like
+        The time series data
+    motifsets: 2d array-like
+        The found motif sets for plotting
+    motif_length: int
+        The motif length found.
+    method_names: array-like
+        Names of the method to plot
+    ground_truth: pd.Series
+        Ground-truth information as pd.Series.
+    grid_dim: int
+        The dimensionality of the grid (number of columns)
+    plot_index: int
+        Plots only the passed methods in the given order
+    """
+
+    # convert to numpy array
+    _, data_raw = ml._pd_series_to_numpy(data)
     D_full = ml.compute_distances_full(data_raw, motif_length)
     indices = np.arange(len(motifsets))
 
@@ -532,10 +645,9 @@ def plot_all_competitors(
              for motiflet_pos in motifsets]
 
     plot_grid_motiflets(
-        name, data, motifsets, indices,
-        dists, motif_length, ds_name=name,
-        # method_name=prefix,
-        font_size=26, 
+        ds_name, data, motifsets, indices,
+        dists, motif_length,
+        font_size=26,
         method_names=method_names,
         ground_truth=ground_truth,
         color_palette=color_palette,
@@ -544,17 +656,35 @@ def plot_all_competitors(
 
 def plot_competitors(
         data,
-        name,
+        ds_name,
         motifsets,
         motif_length,
         prefix="",
         filter=True,
-        target_cardinalities=None,
         ground_truth=None):
+    """Plots motif sets of a single competitor method.
+
+    Parameters
+    ----------
+    data: array-like
+        The time series data
+    ds_name: String
+        The name of the time series
+    motifsets: array-like
+        The motifset for plotting
+    motif_length: int
+        The motif length found.
+    prefix: String
+        The method name
+    filter: bool, default=True
+        filter overlapping motifs
+    ground_truth: pd.Series
+        Ground-truth information as pd.Series.
+
+    """
+
     # convert to numpy array
-    data_raw = data
-    if isinstance(data, pd.Series):
-        data_raw = data.to_numpy()
+    _, data_raw = ml._pd_series_to_numpy(data)
 
     D_full = ml.compute_distances_full(data_raw, motif_length)
 
@@ -569,31 +699,32 @@ def plot_competitors(
     elbow_points = np.arange(len(motifsets_filtered))
 
     if filter:
-        elbow_points = ml.filter_unqiue(elbow_points, motifsets_filtered, motif_length)
+        elbow_points = ml._filter_unique(elbow_points, motifsets_filtered, motif_length)
 
     dists = [ml.get_pairwise_extent(D_full, motiflet_pos, upperbound=np.inf)
              for motiflet_pos in motifsets_filtered]
 
     plot_grid_motiflets(
-        name, data, motifsets_filtered, elbow_points,
-        dists, motif_length, ds_name=name, method_name=prefix, 
+        ds_name, data, motifsets_filtered, elbow_points,
+        dists, motif_length, method_name=prefix,
         ground_truth=ground_truth)
 
     return motifsets_filtered[elbow_points]
 
+
 def format_key(e):
     key = ""
     if e > 0:
-        key = "+"+str(e*100) + "%"
+        key = "+" + str(e * 100) + "%"
     elif e < 0:
-        key = str(e*100) + "%"
+        key = str(e * 100) + "%"
     return key
 
 
 def to_df(motif_sets, method_name, df, df2=None):
     df_all_1 = pd.DataFrame()
     df_all_2 = pd.DataFrame()
-    for key in motif_sets:       
+    for key in motif_sets:
         ms_set_finder = motif_sets[key]
         df_all_1[method_name + " Top-1 " + key] = [ms_set_finder[-1]]
         df[method_name + " Top-1 " + key] = [ms_set_finder[-1]]
@@ -601,11 +732,11 @@ def to_df(motif_sets, method_name, df, df2=None):
         if df2 is not None:
             df_all_2[method_name + " Top-2 " + key] = [ms_set_finder[-2]]
             df2[method_name + " Top-2 " + key] = [ms_set_finder[-2]]
-    
+
     if df2 is not None:
         df_all = (pd.concat([df_all_1, df_all_2], axis=1)).T
-    else: 
+    else:
         df_all = df_all_1.T
 
-    df_all.rename(columns={0:"offsets"}, inplace=True)    
+    df_all.rename(columns={0: "offsets"}, inplace=True)
     return df_all
